@@ -4,8 +4,7 @@ import re
 import subprocess
 import logging
 
-import mini_buildd.call
-from mini_buildd.files import Dir, PerlModule, ShScript, BashScript, AutoScript, SourcesList
+from mini_buildd import util, config, call, files
 
 LOG = logging.getLogger(__name__)
 
@@ -26,97 +25,75 @@ APT_GET = "apt-get --quiet --yes --option=APT::Install-Recommends=false --option
 APT_TRUSTED_GPGD = "/etc/apt/trusted.gpg.d"
 APT_CONFD = "/etc/apt/apt.conf.d"
 
-#: Helper to compute CONFIG_DIR in bash code
-BASH_CONFIG_DIR = f"${{0%/*/*}}/{CONFIG_DIR}"
+#: Safe canonized paths
+BASH_VARS = """\
+MBD_SCRIPT="$(realpath "${0}")"  # Note: Older versions of realpath (<=wheezy) did not support options, just avoid them
+MBD_BASE="${MBD_SCRIPT%/*/*}"\
+"""
 
 
-class Blocks(Dir):
+class Blocks(files.Dir):
     def __init__(self, type_):
         super().__init__()
         self.type = type_
-
-    def extra_option(self, top=False):
-        return f"Sbuild-{self.type.capitalize()}-Blocks{'-Top' if top else ''}"
-
-    def extra_options(self):
-        return [self.extra_option(top=True), self.extra_option()]
+        self.name = f"Sbuild-{self.type.capitalize()}-Blocks"
+        self.default = ""
 
     def usage(self):
-        ml = max(len(name) for name in self.extra_options())  # "max length" helper
-        header = (f"{self.extra_option(top=True):<{ml}}: Blocks before the automated {self.type}.\n"
-                  f"{self.extra_option(top=False):<{ml}}: Blocks after the automated {self.type}.\n\n"
+        default = self.default.strip()
+        header = (f"{self.name}: <block> ...: Ordered sbuild {self.type} blocks (space-separated)\n\n"
+                  f"Default if not given: {self.default}\n\n"
                   "Available blocks:\n\n")
-        ml = max(len(name) for name in self.keys())
-        return header + "\n".join([f"{name:<{ml}}: {block.description}" for name, block in self.items()])
+        ml = max(len(name) for name in self.keys())  # "max length" formatting helper
+        return header + "\n".join([f"{name:<{ml}}: [{'default' if name in default else 'extra'}] {block.description}" for name, block in self.items()])
 
     def validate(self, extra_option):
-        """Validate extra option value from user space (string, space separated). Return extra option value as list."""
+        """Validate extra option value from user space (string, space separated), return extra option value as list"""
         eol = extra_option.split()  # extra option list
         unknown = set(eol) - set(self.keys())
         if unknown:
-            raise mini_buildd.HTTPBadRequest(f"Unknown sbuild {self.type} block(s): {' '.join(unknown)} (check 'Extra Options' configurations in Distributions)")
+            raise util.HTTPBadRequest(f"Unknown sbuild {self.type} block(s): {' '.join(unknown)} (check 'Extra Options' configurations in Distributions)")
         return eol
 
-    def line(self, extra_option, top=False):
+    def line(self, extra_option):
         self.validate(extra_option)
-        return f"{self.extra_option(top=top)}: {extra_option}"
-
-    def validate_all(self, get):
-        for extra_option in self.extra_options():
-            self.validate(get(extra_option, ""))
+        return f"{self.name}: {extra_option}"
 
 
 class ConfigBlocks(Blocks):
     def __init__(self):
         super().__init__("config")
+        self.default = "set-default-path ccache"
 
-    def default(self):
-        return f"{self.extra_option(top=False)}: ccache"
-
-    def configure(self, file_, get, top):
-        """Add configured blocks to provided file (space separated config option is auto-magically received via provided getter)."""
-        for block in self.validate(get(self.extra_option(top=top), "")):
+    def configure(self, file_, blocks):
+        """Add configured blocks to provided file"""
+        for block in self.validate(blocks):
             file_.add(self[block].get(snippet=True))
 
 
 class SetupBlocks(Blocks):
     def __init__(self):
         super().__init__("setup")
+        self.default = "apt-setup apt-update ccache eatmydata"
 
-    def default(self):
-        return f"{self.extra_option(top=False)}: ccache eatmydata"
-
-    def configure(self, dir_, get, top):
-        """Add configured blocks to provided dir (space separated config option is auto-magically received via provided getter)."""
-        dir_.add_dir(self, name_filter=self.validate(get(self.extra_option(top=top), "")))
+    def configure(self, dir_, blocks):
+        """Add configured blocks to provided dir"""
+        for block in self.validate(blocks):
+            dir_.add(self[block])
 
 
 CONFIG_BLOCKS = ConfigBlocks()
 
-CONFIG_BLOCKS.add(PerlModule(
+CONFIG_BLOCKS.add(files.PerlModule(
     "set-default-path",
-    description="auto: Always set $path, so config blocks may add to it (this is sbuild's default path)",
+    description="Explicitly set $path (to sbuild's default value), so later config blocks may add to it",
     snippet="""\
 $path = '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games';
 """))
 
-CONFIG_BLOCKS.add(PerlModule(
-    "no-apt-update",
-    description="auto: Disable sbuild's internal apt update (we do a custom update later via chroot-setup commands)",
-    snippet="""\
-$apt_update = 0;
-"""))
-
-CONFIG_BLOCKS.add(PerlModule(
-    "apt-allow-unauthenticated",
-    description="auto(if configured): Allow unauthenticated apt toggle (from buildrequest)",
-    snippet="""\
-$apt_allow_unauthenticated = 1;
-"""))
-
-CONFIG_BLOCKS.add(PerlModule(
+CONFIG_BLOCKS.add(files.PerlModule(
     "ccache",
-    description="Enable ccache",
+    description="Update $path and $$build environment to support ccache",
     snippet="""\
 $path = "/usr/lib/ccache:$path";
 $$build_environment{'CCACHE_DIR'} = '%LIBDIR%/.ccache';
@@ -124,67 +101,55 @@ $$build_environment{'CCACHE_DIR'} = '%LIBDIR%/.ccache';
 
 SETUP_BLOCKS = SetupBlocks()
 
-SETUP_BLOCKS.add(ShScript(
-    "show-sbuildrc",
-    description="auto-top: Print used '.sbuildrc' (FYI)",
-    snippet=f"""\
-cat "{BASH_CONFIG_DIR}/../.sbuildrc"
-"""))
-
-SETUP_BLOCKS.add(BashScript(
-    "apt-keys",
-    description=f"auto-top: Copy APT keys from buildrequest to ``{APT_TRUSTED_GPGD}``",
-    snippet=f"""\
-mkdir -p -v "{APT_TRUSTED_GPGD}"
-cp -v "{BASH_CONFIG_DIR}/{CONFIG_APT_KEYS}" "{APT_TRUSTED_GPGD}/mini-buildd-buildrequest.asc"
-"""))
-
-SETUP_BLOCKS.add(ShScript(
-    "apt-allow-unauthenticated",
-    description="auto-top(if configured): Allow unauthenticated APT",
-    snippet=f"""
-printf 'APT::Get::AllowUnauthenticated \"true\";\\n' >'{APT_CONFD}/10mbd-allow-unauthenticated'
-"""))
-
-SETUP_BLOCKS.add(BashScript(
-    "apt-setup-https",
-    description="auto(if any https sources): Enable APT https support",
-    snippet=f"""\
-{APT_GET} install ca-certificates apt-transport-https
-cp -v "{BASH_CONFIG_DIR}/{CONFIG_SSL_CERT}" /usr/local/share/ca-certificates/mini-buildd-repo.crt
-/usr/sbin/update-ca-certificates
-"""))
-
-SETUP_BLOCKS.add(ShScript(
+SETUP_BLOCKS.add(files.BashScript(
     "apt-setup",
-    description="auto: Setup APT sources and preferences",
+    description="Sets up APT for buildrequest (APT keys, optional https, sources and preferences)",
     snippet=f"""\
-cp -v "{BASH_CONFIG_DIR}/{CONFIG_APT_SOURCES_LIST}" /etc/apt/sources.list
+{BASH_VARS}
+
+mkdir -p -v "{APT_TRUSTED_GPGD}"
+cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_KEYS}" "{APT_TRUSTED_GPGD}/mini-buildd-buildrequest.asc"
+
+if grep 'https://' "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_SOURCES_LIST}"; then
+  {APT_GET} install ca-certificates apt-transport-https
+  if [ -s "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_SSL_CERT}" ]; then
+    cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_SSL_CERT}" /usr/local/share/ca-certificates/mini-buildd-repo.crt
+    /usr/sbin/update-ca-certificates
+  fi
+fi
+
+cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_SOURCES_LIST}" /etc/apt/sources.list
 cat /etc/apt/sources.list
 
-cp -v "{BASH_CONFIG_DIR}/{CONFIG_APT_PREFERENCES}" /etc/apt/preferences
+cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_PREFERENCES}" /etc/apt/preferences
 cat /etc/apt/preferences
 """))
 
-SETUP_BLOCKS.add(ShScript(
+SETUP_BLOCKS.add(files.ShScript(
     "apt-update",
-    description="auto: Update APT",
+    description="Update APT",
     snippet=f"""\
-{APT_GET} update
-apt-cache policy
+for N in $(seq 5); do
+    if {APT_GET} update; then
+       apt-cache policy
+       exit 0
+    fi
+    sleep ${{N}}
+done
+exit 1
 """))
 
 # All below need explicit opt-in
-SETUP_BLOCKS.add(ShScript(
+SETUP_BLOCKS.add(files.ShScript(
     "ccache",
-    description="Enables use of ccache (for builds on the same instance) -- also needs the 'ccache' config block",
+    description="Installs ccache (together with the 'ccache' config block, this enables ccache for builds on the same instance)",
     snippet=f"""\
 {APT_GET} install ccache
 """))
 
-SETUP_BLOCKS.add(ShScript(
+SETUP_BLOCKS.add(files.ShScript(
     "eatmydata",
-    description="Try to minimize to-disk-syncs -- may improve speed",
+    description="Install eatmydata and activate via LD_PRELOAD when possible (may improve speed)",
     snippet=f"""\
 if {APT_GET} install eatmydata; then
     # eatmydata <= 26: Just one 'deb', and not multiarched
@@ -203,32 +168,63 @@ else
 fi
 """))
 
-SETUP_BLOCKS.add(BashScript(
-    "apt-no-check-valid-until",
-    description="compat(< stretch): Disable APT's 'Check-Valid-Until' globally (only needed where this option can't be given as apt line option)",
+
+SETUP_BLOCKS.add(files.BashScript(
+    "show-sbuildrc",
+    description="Print generated '.sbuildrc' (informational only)",
     snippet=f"""\
-printf "Acquire::Check-Valid-Until \\"false\\";\\n" >"{APT_CONFD}/10mbd-no-check-valid-until"
+{BASH_VARS}
+
+cat "${{MBD_BASE}}/{CONFIG_DIR}/../.sbuildrc"
 """))
 
-SETUP_BLOCKS.add(BashScript(
-    "apt-binary-keys",
-    description="compat(<= jessie, <= yakketti): Convert APT keys to binary",
+
+SETUP_BLOCKS.add(files.BashScript(
+    "apt-key-binary",
+    description="Convert APT keys to binary (needed for Debian <= jessie, Ubuntu <= yakketti) -- place between ``apt-setup`` and ``apt-update``",
     snippet=f"""\
 {APT_GET} update
 {APT_GET} install gnupg
 gpg --dearmor "{APT_TRUSTED_GPGD}/mini-buildd-buildrequest.asc"
 """))
 
-SETUP_BLOCKS.add(BashScript(
+SETUP_BLOCKS.add(files.BashScript(
     "apt-key-add",
-    description="compat(<= squeeze): Add APT keys via ``apt-key add``",
+    description="Add APT keys via deprecated ``apt-key add`` (needed for Debian <= squeeze) -- place between ``apt-setup`` and ``apt-update``",
     snippet=f"""\
-apt-key add "{BASH_CONFIG_DIR}/{CONFIG_APT_KEYS}"
+{BASH_VARS}
+
+apt-key add "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_KEYS}"
 """))
 
-SETUP_BLOCKS.add(ShScript(
+
+def _setup_blocks_apt_option(name, option, desc):
+    """Add a script that sets an APT option"""
+    SETUP_BLOCKS.add(files.ShScript(
+        name,
+        description=f'Set APT option "{option}" ({desc}) -- place before ``apt-setup``',
+        snippet=f'printf "%s\\n" "{option};" >"{APT_CONFD}/00mbd-{name}"\n'))
+
+
+_setup_blocks_apt_option("apt-no-check-valid-until", "Acquire::Check-Valid-Until false", "needed when Release files are expired")
+_setup_blocks_apt_option("apt-allow-unauthenticated", "APT::Get::AllowUnauthenticated true", "needed when APT keys are expired")
+_setup_blocks_apt_option("apt-force-yes", "APT::Get::Force-Yes true", "compat(urold)")
+_setup_blocks_apt_option("apt-no-cache", "Acquire::http::No-Cache true", "compat(urold)")
+_setup_blocks_apt_option("apt-no-pipelining", "Acquire::http::Pipeline-Depth 0", "compat(urold)")
+
+
+SETUP_BLOCKS.add(files.ShScript(
+    "apt-clear",
+    description="Try to clear any redundant APT data in ``/var/`` (experimental: can make APT work again if stuck)",
+    snippet="""\
+rm -v -r -f /var/lib/apt/lists/*
+mkdir /var/lib/apt/lists/partial  # <= in lenny, apt would fail if missing
+apt-get clean
+"""))
+
+SETUP_BLOCKS.add(files.ShScript(
     "bash-devices",
-    description="compat(bash < 3.0-17 (2005)): Fixup /dev/fd|stdin|stdout|stderr. Needed for *building* bash only. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=327477",
+    description="Fixup /dev/fd|stdin|stdout|stderr. Needed for *building* bash only. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=327477 (compat(bash < 3.0-17 (2005))",
     snippet="""\
 [ -e /dev/fd ] || ln -sv /proc/self/fd /dev/fd
 [ -e /dev/stdin ] || ln -sv fd/0 /dev/stdin
@@ -236,9 +232,9 @@ SETUP_BLOCKS.add(ShScript(
 [ -e /dev/stderr ] || ln -sv fd/2 /dev/stderr
 """))
 
-SETUP_BLOCKS.add(BashScript(
+SETUP_BLOCKS.add(files.BashScript(
     "schroot-shm",
-    description="compat(schroot < 1.6.10-3): Fixup /dev/shm mount. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=728096",
+    description="Fixup /dev/shm mount. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=728096 (compat(schroot < 1.6.10-3))",
     snippet="""\
 # Pre 1.99.19, this used to live here: /etc/schroot/setup.d/15mini-buildd-workarounds
 
@@ -271,31 +267,15 @@ mkdir -v -p "/dev/shm"
 mount -v -ttmpfs none "/dev/shm"
 """))
 
-SETUP_BLOCKS.add(ShScript(
+SETUP_BLOCKS.add(files.ShScript(
     "sun-java6-license",
-    description="compat(< 2011): Accept sun-java6 licence (debconf) so we can build-depend on it (sun-java is no longer part of Debian since 2011)",
+    description="Accept sun-java6 licence (debconf) so we can build-depend on it (sun-java is no longer part of Debian since 2011)",
     snippet="""\
 echo "sun-java6-bin shared/accepted-sun-dlj-v1-1 boolean true" | debconf-set-selections --verbose
 echo "sun-java6-jdk shared/accepted-sun-dlj-v1-1 boolean true" | debconf-set-selections --verbose
 echo "sun-java6-jre shared/accepted-sun-dlj-v1-1 boolean true" | debconf-set-selections --verbose
 """))
 
-SETUP_BLOCKS.add(ShScript(
-    "apt-clear",
-    description="compat,experimental: Try to clear any redundant APT data in ``/var/`` (can make APT work again if stuck)",
-    snippet="""\
-rm -v -r -f /var/lib/apt/lists/*
-mkdir /var/lib/apt/lists/partial  # <= in lenny, apt would fail if missing
-apt-get clean
-"""))
-
-SETUP_BLOCKS.add(ShScript(
-    "apt-urold",
-    description="compat,experimental(< wheezy): Try some hard configs for urold releases (squeeze or older) to make APT work",
-    snippet=f"""\
-printf "%s\\n" "Acquire::http::No-Cache true;" "Acquire::http::Pipeline-Depth 0;" "APT::Get::Force-Yes true;" >"{APT_CONFD}/00mbd-urold"
-"""))
-
 
 class SBuild():
     def __init__(self, breq):
@@ -305,28 +285,16 @@ class SBuild():
         self.config_path = self.breq.builds_path.new_sub([CONFIG_DIR])
         self.sbuildrc_file_path = self.breq.builds_path.join(".sbuildrc")
         self.buildlog_file_path = self.breq.builds_path.join(self.breq.dfn.buildlog(self.breq["architecture"]))
-        self.apt_allow_unauthenticated = breq.cget('Apt-Allow-Unauthenticated', "0") == "1"
 
         self.call = None
 
         #
         # Generate .sbuildrc
         #
-        sbuildrc = PerlModule(".sbuildrc", placeholders={"LIBDIR": mini_buildd.config.ROUTES["libdir"].path.new_sub([str(self.breq.dist.codename), self.breq["architecture"]], create=True).full})
-
-        # Top, auto
-        sbuildrc.add(CONFIG_BLOCKS["set-default-path"].get(snippet=True))
-
-        # Top, from distribution config
-        CONFIG_BLOCKS.configure(sbuildrc, self.breq.cget, top=True)
-
-        # auto
-        sbuildrc.add(CONFIG_BLOCKS["no-apt-update"].get(snippet=True))
-        if self.apt_allow_unauthenticated:
-            sbuildrc.add(CONFIG_BLOCKS["apt-allow-unauthenticated"].get(snippet=True))
+        sbuildrc = files.PerlModule(".sbuildrc", placeholders={"LIBDIR": config.ROUTES["libdir"].path.new_sub([str(self.breq.dist.codename), self.breq["architecture"]], create=True).full})
 
         # From distribution config
-        CONFIG_BLOCKS.configure(sbuildrc, self.breq.cget, top=False)
+        CONFIG_BLOCKS.configure(sbuildrc, self.breq.cget(CONFIG_BLOCKS.name, CONFIG_BLOCKS.default))
 
         # Custom script from distribution config
         sbuildrc.add_file(self.config_path.join(CONFIG_SBUILDRC_SNIPPET), description="Custom sbuild config (from Distribution)")
@@ -336,30 +304,15 @@ class SBuild():
         #
         # Generate setup.d/
         #
-        setup_d = Dir(self.breq.builds_path.join(SETUP_D_DIR), enumerate_file_names=0)
-
-        # Top, auto
-        setup_d.add(SETUP_BLOCKS["show-sbuildrc"])
-        setup_d.add(SETUP_BLOCKS["apt-keys"])
-        if self.apt_allow_unauthenticated:
-            setup_d.add(SETUP_BLOCKS["apt-allow-unauthenticated"])
-
-        # Top, from distribution config
-        SETUP_BLOCKS.configure(setup_d, self.breq.cget, top=True)
-
-        # auto
-        if SourcesList.has_https(self.config_path.join(CONFIG_APT_SOURCES_LIST)):
-            setup_d.add(SETUP_BLOCKS["apt-setup-https"])
-        setup_d.add(SETUP_BLOCKS["apt-setup"])
-        setup_d.add(SETUP_BLOCKS["apt-update"])
+        setup_d = files.Dir(self.breq.builds_path.join(SETUP_D_DIR), enumerate_file_names=0)
 
         # From distribution config
-        SETUP_BLOCKS.configure(setup_d, self.breq.cget, top=False)
+        SETUP_BLOCKS.configure(setup_d, self.breq.cget(SETUP_BLOCKS.name, SETUP_BLOCKS.default))
 
         # Custom script from distribution config
         custom_buildrequest = self.config_path.join(CONFIG_CHROOT_SETUP_SCRIPT)
         if os.path.getsize(custom_buildrequest) > 0:
-            setup_d.add(AutoScript(custom_buildrequest))
+            setup_d.add(files.AutoScript(custom_buildrequest))
 
         setup_d.save()
 
@@ -376,10 +329,11 @@ class SBuild():
             self.cmdline += ["--chroot-setup-command", script]
 
         self.cmdline += ["--build-dep-resolver", self.breq.cget("Build-Dep-Resolver"),
-                         "--keyid", mini_buildd.get_daemon().gnupg.get_first_sec_key(),
+                         "--keyid", util.daemon().gnupg.get_first_sec_key(),
                          "--nolog",
                          "--log-external-command-output",
-                         "--log-external-command-error"]
+                         "--log-external-command-error",
+                         "--no-apt-update"]
 
         if self.breq.cget("Arch-All"):
             self.cmdline += ["--arch-all"]
@@ -413,7 +367,7 @@ class SBuild():
         else:
             self.cmdline += ["--no-run-autopkgtest"]
 
-        if "sbuild" in mini_buildd.config.DEBUG:
+        if "sbuild" in config.DEBUG:
             self.cmdline += ["--debug"]
 
         self.cmdline += [self.breq.dfn.dsc()]
@@ -422,7 +376,7 @@ class SBuild():
 
     def run(self, bres):
         """
-        Run sbuild && update buildresult from build log.
+        Run sbuild && update buildresult from build log
 
         .. note:: This will iterate all lines of the build log,
                   and parse out the selection of sbuild's summary status
@@ -433,13 +387,13 @@ class SBuild():
                   better way to get these values.
         """
         # Run sbuild
-        with mini_buildd.fopen(self.buildlog_file_path, "w+") as buildlog, subprocess.Popen(
+        with util.fopen(self.buildlog_file_path, "w+") as buildlog, subprocess.Popen(
                 self.cmdline,
                 cwd=self.breq.builds_path.full,
-                env=mini_buildd.call.taint_env({"HOME": self.breq.builds_path.full,
-                                                "GNUPGHOME": mini_buildd.config.ROUTES["home"].path.join(".gnupg"),
-                                                "DEB_BUILD_OPTIONS": self.breq.cget("Deb-Build-Options", ""),
-                                                "DEB_BUILD_PROFILES": self.breq.cget("Deb-Build-Profiles", "")}),
+                env=call.taint_env({"HOME": self.breq.builds_path.full,
+                                    "GNUPGHOME": config.ROUTES["home"].path.join(".gnupg"),
+                                    "DEB_BUILD_OPTIONS": self.breq.cget("Deb-Build-Options", ""),
+                                    "DEB_BUILD_PROFILES": self.breq.cget("Deb-Build-Profiles", "")}),
                 stdout=buildlog,
                 stderr=subprocess.STDOUT) as self.call:
             LOG.info(f"Building {self.breq} (sbuild process '{self.call.pid}')...")
@@ -447,7 +401,7 @@ class SBuild():
 
         # Update bres from buildlog
         bres.add_file(self.buildlog_file_path)
-        with mini_buildd.fopen(self.buildlog_file_path, errors="replace") as f:
+        with util.fopen(self.buildlog_file_path, errors="replace") as f:
             for line in f:
                 if self.BUILDLOG_STATUS_REGEX.match(line):
                     LOG.debug(f"Build log line detected as build status: {line.strip()}")
